.colorPicker{
	width: 30px;
	height: 30px;
	position: relative;
	clear: both;
	margin: 80px auto 20px;
}

.colorPicker .track {
	background: #EFEFEF url(../img/text-color.png) no-repeat 50% 50%;
	height: 170px;
	width: 170px;
	padding: 10px;
	position: absolute;
	cursor: crosshair;
	float: left;
	left: -71px;
	top: -71px;
	display: none;
	border: 1px solid #ccc;
	z-index: 10;
	-webkit-border-radius: 150px;
	-moz-border-radius: 150px;
	border-radius: 150px;

}

.colorPicker .color {
	width: 27px;
	height: 27px;
	padding: 0px;
	border: 1px solid #ccc;
	display: block;
	position: relative;
	z-index: 11;
	background-color: #EFEFEF;
	-webkit-border-radius: 27px;
	-moz-border-radius: 27px;
	border-radius: 27px;
	cursor: pointer;
}

.colorPicker .colorInner {
	width: 25px;
	height: 25px;
	-webkit-border-radius: 27px;
	-moz-border-radius: 27px;
	border-radius: 27px;
}

.colorPicker .dropdown {
	list-style: none;
	display: none;
	width: 27px;
	position: absolute;
	top: 28px;
	border: 1px solid #ccc;
	left: 0;
	z-index: 1000;
}

.colorPicker .dropdown li{
	height: 25px;
	cursor: pointer;
}

